/*
 * @file libpixy2.c
 *
 * @author Jon McPhalen
 *
 * @copyright
 * Copyright (c) Jon McPhalen, 2020. All Rights MIT Licensed.
 *
 * @brief Demonstrates serial communications to the Pixy2 camera and the
 * ability to monitor Color Connected Components.
 */

#include "simpletools.h"
#include "pixy2.h"

#define  RX2  5                                                 // serial pins
#define  TX2  4

#define  SIG_MAP  CCC_SIG1 | CCC_SIG2 | CCC_SIG3                // monitor Signatures 1, 2, and 3
#define  CC_MAP   CCC_COLOR_CODES | SIG_MAP                     // monitor color codes for CC sigs 1, 2, and 3

uint16_t frameWidth;                                            // width of Pixy frame
uint16_t frameHeight;                                           // height of Pixy frame

block_t   b;                                                    // target (block) values


void showVersion();                                             // function prototypes for demo
void showResolution(void);
void showFrameRate(void);
void showSignatures(void);
void showColorCodes(void);
void showColorDetect(uint8_t mode);
void showError(int32_t code);


int main()
{
  uint32_t ec;
  int k;

  // Note: Pixy can take up to five seconds to be ready.
  //       Insert delay here if required.

  ec = pixy2_start(RX2, TX2, 19200);
  if (ec != RESULT_OK) {
    print("Error: Unable to create serial connection\n\n");
    while (1) { ; }
  }  
  else
    print("Pixy2 Serial Test\n\n");
    
  showVersion();

  showResolution();

  showFrameRate();
  
  while (1) {
    print("Select Demo:\n");
    print("[1] Track Color Signatures\n");
    print("[2] Track Color Codes\n");
    print("[3] Read color (raw)\n");
    print("[4] Read color (saturated)\n\n");
    k = getChar();
    if ((k >= '1') && (k <= '4'))
      break;
    else
      print("-- bad selection\n\n");
  }    
  
  while (1) {
    switch (k) {
      case '1':
        showSignatures();
        break;
        
      case '2':
        showColorCodes();
        break;      
      
      case '3':
        showColorDetect(0);
        break;      
    
      case '4':
        showColorDetect(1);
        break;
    }           
    pause(1000);
  }
}


void showVersion()
{
  int ec;

  ec = pixy2_getVersion();

  if (ec == RESULT_OK) {
    print("Hardware = %d\n", pixy2_hardware());
    print("Firmware = %d.%d.%d ", pixy2_fwMajor(), pixy2_fwMinor(), pixy2_fwBuild());
    print("(%s)\n\n", pixy2_fwType());
  }
  else
    showError(ec);
}


void showResolution()
{
  int ec;

  ec = pixy2_getResolution(&frameWidth, &frameHeight);

  if (ec == RESULT_OK) 
    print("X Pixels = %d\nY Pixels = %d\n\n", frameWidth, frameHeight);
  else
    showError(ec);
}


void showFrameRate()
{
  int fps = pixy2_getFPS();

  if (fps >= 0) 
    print("Frames/s = %d\n\n", fps);
  else
    showError(fps);
}


void showSignatures()
{
  int32_t nsigs;
  uint8_t i;
  
  nsigs = pixy2_getBlocks(SIG_MAP, MAX_BLOCKS);
  
  if (nsigs < RESULT_OK)  
    showError(nsigs);
  else {
    print("%d blocks detected\n\n", nsigs);
    if (nsigs > 0) {
      for (i = 0; i < nsigs; i++) {
        pixy2_extractBlock(i, &b.signature);
        print("sig: %1d  x: %3d  y: %3d  ", b.signature, b.xpos, b.ypos);
        print("w: %3d  h: %3d\n", b.width, b.height);
      }
      print("\n");
    }
  }    
}

  
void showColorCodes()
{
  int32_t nccodes;
  uint8_t i;
  
  nccodes = pixy2_getBlocks(CC_MAP, MAX_BLOCKS);
  
  if (nccodes < RESULT_OK)  
    showError(nccodes);
  else {
    print("%d color codes detected\n\n", nccodes);
    if (nccodes > 0) {
      for (i = 0; i < nccodes; i++) {
        pixy2_extractBlock(i, &b.signature);
        // printf required for octal
        printf("ccode: %3o  x: %3d  y: %3d  ", b.signature, b.xpos, b.ypos);
        printf("w: %3d  h: %3d  a: %+3d\n", b.width, b.height, b.angle);
      }
      print("\n");
    }
  }    
}

  
void showColorDetect(uint8_t mode)
{
  int32_t color;
  
  color = pixy2_getRGB(frameWidth/2, frameHeight/2, mode);
  
  if (color < RESULT_OK)  
    showError(color);
  else 
    print("Color = %06x\n", color);
}  


void showError(int32_t code)
{
  print("Pixy error: %d\n", code);
  // while (1) { ; }
}


/*
 * TERMS OF USE: MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
